@@ -1,20 +1,30 @@
-//
-// coff.c - Common Object File Format (COFF) support
-//
-//	This file was developed for the avra assembler in order to produce COFF output files 
-//	for use with the Atmel AVR Studio.  The Lean C Compiler (LCC) debugging stabs 
-//	output was used as input to the assembler.  The information used to develop this file
-//	was obtained from various sources on the Internet, most notably, the Free Software Foundation,
-//	The "stabs" debug format, ??? Chapter 7: Common Object File Format (COFF), 
-//
-//	This software has absolutely no warrantee!  The money you paid for this will be 
-//	promptly refunded if not fully satisfied.
-//
-//	Beta release 1/20/2000 by Bob Harris
-//
-//	This software has not been fully tested and probably has a few software deficiencies.
-//	Some software support may be possible by sending a problem description report to 
-//	rth@mclean.sparta.com
+/***********************************************************************
+ *
+ *  avra - Assembler for the Atmel AVR microcontroller series
+ *
+ *  Copyright (C) 1998-2004 Jon Anders Haugum, Tobias Weber
+ *
+ * coff.c - Common Object File Format (COFF) support
+ *
+ *	This file was developed for the avra assembler in order to produce COFF output files 
+ *  for use with the Atmel AVR Studio.  The Lean C Compiler (LCC) debugging stabs 
+ *	output was used as input to the assembler.  The information used to develop this file
+ *  was obtained from various sources on the Internet, most notably, the Free Software Foundation,
+ *  The "stabs" debug format, ??? Chapter 7: Common Object File Format (COFF), 
+ *
+ *	This software has absolutely no warrantee!  The money you paid for this will be 
+ *  promptly refunded if not fully satisfied.
+ * 
+ *	Beta release 1/20/2000 by Bob Harris
+ *
+ *	This software has not been fully tested and probably has a few software deficiencies.
+ *  Some software support may be possible by sending a problem description report to 
+ *  rth@mclean.sparta.com
+ *
+ *  Made the recommended change in write_coff_program().
+ *  Fixed an obvious typo in SkipPastDigits().  The if() statement was terminated 
+ *  with a semicolon, which terminated the if(); early.  JEG 4-01-03
+ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -32,36 +42,37 @@
 
 struct FundamentalType {
 
-    char *pString;
+    char const *pString;
     int Type;
     int Size;
 };
 
 struct FundamentalType FundamentalTypes[] = {
-    "null", T_NULL, 0,
-    "void", T_VOID, 0,
-    "char", T_CHAR, 1,
-    "short", T_SHORT, 1,
-    "int", T_INT, 1,
-    "long", T_LONG, 2, 
-    "float", T_FLOAT, 4,
-    "double", T_DOUBLE, 4,
-    "struct", T_STRUCT, 0,
-    "union", T_UNION, 0,
-    "enum",  T_ENUM, 0,
-    "moe", T_MOE, 0,
-    "unsigned char", T_UCHAR, 1,
-    "unsigned short", T_USHORT, 1,
-    "unsigned int", T_UINT, 1,
-    "unsigned long", T_ULONG, 2,
-    "long double", T_LNGDBL, 2,
-    "long long int", T_LONG, 2,
-    "long int", T_LONG, 2,
-    "unsigned long long", T_ULONG, 2, 
-    "signed char", T_CHAR, 1,
-    0, 0
+    {"null", T_NULL, 0},
+    {"void", T_VOID, 0},
+    {"char", T_CHAR, 1},
+    {"short", T_SHORT, 1},
+    {"int", T_INT, 1},
+    {"long", T_LONG, 2}, 
+    {"float", T_FLOAT, 4},
+    {"double", T_DOUBLE, 4},
+    {"struct", T_STRUCT, 0},
+    {"union", T_UNION, 0},
+    {"enum",  T_ENUM, 0},
+    {"moe", T_MOE, 0},
+    {"unsigned char", T_UCHAR, 1},
+    {"unsigned short", T_USHORT, 1},
+    {"unsigned int", T_UINT, 1},
+    {"unsigned long", T_ULONG, 2},
+    {"long double", T_LNGDBL, 2},
+    {"long long int", T_LONG, 2},
+    {"long int", T_LONG, 2},
+    {"unsigned long long", T_ULONG, 2}, 
+    {"signed char", T_CHAR, 1},
+    {0, 0}
 };
 
+
 struct coff_info *ci;
 
 /****************************************************************************************/
@@ -315,7 +326,7 @@ void write_coff_file(struct prog_info *pi){
     /* Section N Header - .data or eeprom */
 
     // Raw Data for Section 1
-    if ( (p = (unsigned char *)FindFirstListObject(&ci->ListOfRawData) ) == 0 ) {
+    if ( (p = FindFirstListObject(&ci->ListOfRawData) ) == 0 ) {
         fprintf(stderr,"\nInternal error - unable to find binary data!");
         return;
     }
@@ -438,7 +449,8 @@ void write_coff_program( struct prog_info *pi, int address, unsigned int data){
 
     /* Coff output keeps track of binary data in memory buffers, address is in bytes not words */
     if ( ci->pRomMemory ) {
-        if ( address <= pi->device->flash_size ) {
+/* JEG	if ( address <= pi->device->flash_size ) {  */  /* JEG 4-23-03 */
+		if ( address <= pi->device->flash_size*2 ) {
             pByte = (unsigned char *)(ci->pRomMemory + address); /* point to low byte in memory */
             *pByte++ = (data & 0xff);   /* low byte */
             *pByte = ((data >> 8) & 0xff); /* high byte */
@@ -447,7 +459,8 @@ void write_coff_program( struct prog_info *pi, int address, unsigned int data){
                 ci->MaxRomAddress = address;   /* keep high water mark */
         } else {
             pi->error_count++;
-            fprintf(stderr, "Error: FLASH address %d exceeds max range %d", address, pi->device->flash_size );
+/* JEG		fprintf(stderr, "Error: FLASH address %d exceeds max range %d", address, pi->device->flash_size ); */
+			fprintf(stderr, "Error: FLASH address %d exceeds max range %d", address, pi->device->flash_size*2 );
         }
     }
 }
@@ -480,14 +493,14 @@ void close_coff_file(struct prog_info *pi, FILE *fp){
 
 /****************************************************************************************/
 
-int parse_stabs( struct prog_info *pi, char *p, int pass ){
+int parse_stabs( struct prog_info *pi, char *p ){
 
     int ok = True;
     int TypeCode, n;
     char *pString, *p2, *p3, *p4, *p5, *pType, *pEnd, *pp, *pJoined;
 
 
-    if ( !GET_ARG(pi->args, ARG_COFF) || ( pass == PASS_1 ) )
+    if ( !GET_ARG(pi->args, ARG_COFF) || ( pi->pass == PASS_1 ) )
         return(True);
 
     /* stabs debugging information is in the form:
@@ -631,7 +644,7 @@ int parse_stabs( struct prog_info *pi, char *p, int pass ){
 }
 /****************************************************************************************/
 
-int parse_stabn( struct prog_info *pi, char *p, int pass ){
+int parse_stabn( struct prog_info *pi, char *p ){
 
     int ok = True;
     int TypeCode /* , LineNumber */, Level;
@@ -648,7 +661,7 @@ int parse_stabn( struct prog_info *pi, char *p, int pass ){
     N_SLINE	0x44		src line: 0,,0,linenumber,address
     */
 
-    if ( !GET_ARG(pi->args, ARG_COFF) || ( pass == PASS_1 ) )
+    if ( !GET_ARG(pi->args, ARG_COFF) || ( pi->pass == PASS_1 ) )
         return(True);
 
     /* Parse the tokens in the stabn line buffer */
@@ -1217,10 +1230,11 @@ int GetStructUnionTagItem( char *p, char **pEnd, char **pName, unsigned short *p
     *pName = p;
     while ( *p && (*p != ':') ) p++; // locate colon
     if ( *p != ':' ) {
-        fprintf(stderr,"\nNo colon found in structure item ->", p);
-        return(False);      
+      fprintf(stderr,"\nNo colon found in structure item -> %s", *pName);
+      return(False);      
     }
     *p++ = 0; // Asciiz
+ 
     /* Stab Type - convert to Coff type at end (after inline assignments */
     if ( GetStabType( p, &StabType, &p ) != True ) {
         fprintf(stderr,"\nInvalid tag type found in structure item -> %s", p);
@@ -1262,8 +1276,8 @@ int GetEnumTagItem( char *p, char **pEnd, char **pEnumName, int *pEnumValue ) {
     *pEnumName = p;
     while ( *p && (*p != ':') ) p++; // locate colon
     if ( *p != ':' ) {
-        fprintf(stderr,"\nNo colon found in enum item ->", p);
-        return(False);      
+      fprintf(stderr,"\nNo colon found in enum item -> %s", *pEnumName);
+      return(False);      
     }
     *p++ = 0; // Asciiz
     *pEnumValue = atoi(p);
@@ -1889,8 +1903,9 @@ char *SkipPastDigits( char *p ){
 
     if ( !p )
         return(p);
-    if ( *p == 0 );
-    return(p);
+/*	if ( *p == 0 );  */  /* JEG 5-01-03 */
+	if ( *p == 0 )
+    return(p);   /* This line s/b indented JEG */
     for ( p--; (*p >= '0') && (*p <= '9') && (*p != 0); p-- );
     return(p);
 }
