@@ -1,6 +1,8 @@
 /***********************************************************************
+ *
  *  avra - Assembler for the Atmel AVR microcontroller series
- *  Copyright (C) 1998-2001 Jon Anders Haugum
+ *
+ *  Copyright (C) 1998-2004 Jon Anders Haugum, Tobias Weber
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,15 +20,17 @@
  *  Boston, MA 02111-1307, USA.
  *
  *
- *  Author of avra can be reached at:
- *     email: jonah@omegav.ntnu.no
- *     www: http://www.omegav.ntnu.no/~jonah/el/avra.html
+ *  Authors of avra can be reached at:
+ *     email: jonah@omegav.ntnu.no, tobiw@suprafluid.com
+ *     www: http://sourceforge.net/projects/avra
  */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+#include <unistd.h> /* B.A. for unlink function */
+
 
 #include "misc.h"
 #include "avra.h"
@@ -34,125 +38,175 @@
 
 
 int open_out_files(struct prog_info *pi, char *filename)
-	{
+{
 	int length;
 	char *buff;
 	time_t tp;
-	int ok; /* flag for coff results */
+	int ok = True; /* flag for coff results */
 
 	length = strlen(filename);
 	buff = malloc(length + 9);
-	if(buff)
-		{
-		strcpy(buff, filename);
-		if(length >= 4)
-			if(!nocase_strcmp(&buff[length - 4], ".asm"))
-				{
-				length -= 4;
-				buff[length] = '\0';
-				}
-		strcpy(&buff[length], ".list");
-		pi->list_file = fopen(buff, "w");
-		if(pi->cseg_count)
-			{
-			strcpy(&buff[length], ".hex");
-			pi->hfi = open_hex_file(buff);
-			strcpy(&buff[length], ".obj");
-			pi->obj_file = open_obj_file(pi, buff);
-			}
-		if(pi->eseg_count)
-			{
-			strcpy(&buff[length], ".eep.hex");
-			pi->eep_hfi = open_hex_file(buff);
-			}
-
-		if(GET_ARG(pi->args, ARG_COFF)){
-			strcpy(&buff[length], ".cof");
-			pi->coff_file = open_coff_file(pi, buff);
-			if( pi->coff_file )
-				ok = True;
-			else
-				ok = False;
-		}
-		else{
-			pi->coff_file = 0;
-			ok = True;
-		}
-
-		free(buff);
-
-		if(pi->list_file && pi->obj_file && (!pi->cseg_count || pi->hfi) && (!pi->eseg_count || pi->eep_hfi) && ok)
-			{
-			if(time(&tp) != -1)
-				fprintf(pi->list_file, "\navra   ver. %d.%d  %s %s\n\n", VERSION, REVISION, filename, ctime(&tp));
-			return(True);
-			}
-		else
-			close_out_files(pi);
-		}
-	else
-		print_msg(pi, MSGTYPE_OUT_OF_MEM, NULL);
-	return(False);
+	if(buff == NULL) {
+	  print_msg(pi, MSGTYPE_OUT_OF_MEM, NULL);
+    return(False);
+  }
+  strcpy(buff, filename);
+  if(length < 4) {
+    printf("Error: wrong input file name\n");
+  }
+  if(!nocase_strcmp(&buff[length - 4], ".asm")) {
+	  length -= 4;
+	  buff[length] = '\0';
 	}
+  //printf("pi->cseg_count = %i\n", pi->cseg_count);
+  //printf("pi->eseg_count = %i\n", pi->eseg_count);
+	
+	/* open files for code output */
+  strcpy(&buff[length], ".hex");
+	if(!(pi->hfi = open_hex_file(buff))) { /* check if open failed */
+    print_msg(pi, MSGTYPE_ERROR, "Could not create output hex file!");
+    ok = False;
+  }
+  strcpy(&buff[length], ".obj");
+  if(!(pi->obj_file = open_obj_file(pi, buff))) {
+    print_msg(pi, MSGTYPE_ERROR, "Could not create object file!");
+    ok = False;
+  }  
+	
+	/* open files for eeprom output */
+  strcpy(&buff[length], ".eep.hex");
+  if(!(pi->eep_hfi = open_hex_file(buff))) {
+		print_msg(pi, MSGTYPE_ERROR, "Could not create eeprom hex file!");
+    ok = False;
+  }  
+  /* coff file is always generated */
+  strcpy(&buff[length], ".cof");
+	pi->coff_file = open_coff_file(pi, buff);
+  /* open list file */
+	if (pi->list_on) {
+    strcpy(buff, GET_ARG(pi->args, ARG_LISTFILE));
+    pi->list_file = fopen(buff, "w");
+    if(pi->list_file == NULL) {
+			print_msg(pi, MSGTYPE_ERROR, "Could not create list file!");
+      ok = False;
+    }
+    /* write list file header */
+    if(time(&tp) != -1) {
+      fprintf(pi->list_file, "\navra   ver. %i.%i.%i %s %s\n\n", VER_MAJOR, VER_MINOR, VER_RELEASE, filename, ctime(&tp));
+    }
+    else {
+			print_msg(pi, MSGTYPE_ERROR, "Timestamp error! check implementation of time() !!!");
+    }  
+  }
+  else {
+    pi->list_file = NULL;
+  }
+  free(buff);
 
+	if(ok)
+	  return(True);
+  else
+	  close_out_files(pi);
+  return(False);
+}
+
+/* delete all output files */
+void unlink_out_files(struct prog_info *pi, char *filename)
+{
+	char *buff;
+	int length;
+
+  close_out_files(pi);
+
+	length = strlen(filename);
+	buff = malloc(length + 9);
+	if(buff == NULL) {
+	  print_msg(pi, MSGTYPE_OUT_OF_MEM, NULL);
+    return;
+  }
+  strcpy(buff, filename);
+  if(!nocase_strcmp(&buff[length - 4], ".asm")) {
+	  length -= 4;
+	  buff[length] = '\0';
+	}
+#if debug == 1
+  printf("unlinking files");
+#endif
+  strcpy(&buff[length], ".hex");
+  unlink(buff);
+  strcpy(&buff[length], ".obj");
+  unlink(buff);
+  strcpy(&buff[length], ".eep.hex");
+  unlink(buff);
+  strcpy(&buff[length], ".cof");
+  unlink(buff);
+  strcpy(&buff[length], ".lst");
+  unlink(buff);
+  strcpy(&buff[length], ".map");
+  unlink(buff);
+}  
 
 void close_out_files(struct prog_info *pi)
-	{
-	if(pi->error_count == 0)
-		{
-		printf("Segment usage:\n"
-		       "   Code:   %7d words (%d bytes)\n"
-		       "   Data:   %7d bytes\n"
-		       "   EEPROM: %7d bytes\n",
-		       pi->cseg_count, pi->cseg_count * 2, pi->dseg_count, pi->eseg_count);
-		}
-	if(pi->hfi) close_hex_file(pi->hfi);
-	if(pi->eep_hfi) close_hex_file(pi->eep_hfi);
-	if(pi->list_file)
-		{
-		if(pi->error_count == 0)
-			fprintf(pi->list_file, "\nAssembly complete with no errors.\n");
-		fclose(pi->list_file);
-		}
-	if(pi->obj_file) close_obj_file(pi, pi->obj_file);
-	if(pi->coff_file) close_coff_file(pi, pi->coff_file);
+{
+  char stmp[2048];
+  
+	if(pi->error_count == 0) {
+		sprintf(stmp,
+      "Segment usage:\n"
+		  "   Code      :   %7d words (%d bytes)\n"
+		  "   Data      :   %7d bytes\n"
+		  "   EEPROM    :   %7d bytes\n",
+		  pi->cseg_count, pi->cseg_count * 2, pi->dseg_count, pi->eseg_count);
+    printf("%s", stmp);
 	}
+	if(pi->hfi)
+    close_hex_file(pi->hfi);
+	if(pi->eep_hfi)
+    close_hex_file(pi->eep_hfi);
+	if(pi->list_file) {
+	  fprintf(pi->list_file, "\n\n%s", stmp);
+		if(pi->error_count == 0)
+			fprintf(pi->list_file, "\nAssembly completed with no errors.\n");
+		fclose(pi->list_file);
+	}
+	if(pi->obj_file)
+    close_obj_file(pi, pi->obj_file);
+	if(pi->coff_file)
+    close_coff_file(pi, pi->coff_file);
+}
 
 
 struct hex_file_info *open_hex_file(char *filename)
-	{
+{
 	struct hex_file_info *hfi;
 
 	hfi = calloc(1, sizeof(struct hex_file_info));
-	if(hfi)
-		{
+	if(hfi) {
 		hfi->segment = -1;
 		hfi->fp = fopen(filename, "wb");
-		if(!hfi->fp)
-			{
+		if(!hfi->fp) {
 			close_hex_file(hfi);
 			hfi = NULL;
-			}
 		}
-	return(hfi);
 	}
+	return(hfi);
+}
 
 
 void close_hex_file(struct hex_file_info *hfi)
-	{
-	if(hfi->fp)
-		{
+{
+	if(hfi->fp) {
 		if(hfi->count != 0)
 		  do_hex_line(hfi);
 		fprintf(hfi->fp, ":00000001FF\x0d\x0a");
 		fclose(hfi->fp);
-		}
-	free(hfi);
 	}
+	free(hfi);
+}
 
 
 void write_ee_byte(struct prog_info *pi, int address, unsigned char data)
-	{
+{
 	if((pi->eep_hfi->count == 16) || ((address != (pi->eep_hfi->linestart_addr + pi->eep_hfi->count)) && (pi->eep_hfi->count != 0)))
 		do_hex_line(pi->eep_hfi);
 	if(pi->eep_hfi->count == 0)
@@ -161,15 +215,13 @@ void write_ee_byte(struct prog_info *pi, int address, unsigned char data)
 
 	if(pi->coff_file)
 	        write_coff_eeprom(pi, address, data);
-	}
-
+}
 
 void write_prog_word(struct prog_info *pi, int address, int data)
-	{
+{
 	write_obj_record(pi, address, data);
 	address *= 2;
-	if(pi->hfi->segment != (address >> 16))
-	{
+	if(pi->hfi->segment != (address >> 16))	{
 	  if(pi->hfi->count != 0)
 	    do_hex_line(pi->hfi);
 	  pi->hfi->segment = address >> 16;
@@ -189,35 +241,33 @@ void write_prog_word(struct prog_info *pi, int address, int data)
 
 	if(pi->coff_file != 0)
 	        write_coff_program(pi, address, data);
-	}
+}
 
 
 void do_hex_line(struct hex_file_info *hfi)
-	{
+{
 	int i;
 	unsigned char checksum = 0;
 
 	fprintf(hfi->fp, ":%02X%04X00", hfi->count, hfi->linestart_addr & 0xffff);
 	checksum -= hfi->count + ((hfi->linestart_addr >> 8) & 0xff) + (hfi->linestart_addr & 0xff);
-	for(i = 0; i < hfi->count; i++)
-		{
+	for(i = 0; i < hfi->count; i++)	{
 		fprintf(hfi->fp, "%02X", hfi->hex_line[i]);
 		checksum -= hfi->hex_line[i];
-		}
+	}
 	fprintf(hfi->fp, "%02X\x0d\x0a", checksum);
 	hfi->count = 0;
-	}
+}
 
 
 FILE *open_obj_file(struct prog_info *pi, char *filename)
-	{
+{
 	int i;
 	FILE *fp;
 	struct include_file *include_file;
 
 	fp = fopen(filename, "wb");
-	if(fp)
-		{
+	if(fp) {
 		i = pi->cseg_count * 9 + 26;
 		fputc((i >> 24) & 0xff, fp);
 		fputc((i >> 16) & 0xff, fp);
@@ -235,27 +285,26 @@ FILE *open_obj_file(struct prog_info *pi, char *filename)
 		fputc(i, fp);
 		fprintf(fp, "AVR Object File");
 		fputc('\0', fp);
-		}
-	return(fp);
 	}
+	return(fp);
+}
 
 
 void close_obj_file(struct prog_info *pi, FILE *fp)
-	{
+{
 	struct include_file *include_file;
 
-	for(include_file = pi->first_include_file; include_file; include_file = include_file->next)
-		{
+	for(include_file = pi->first_include_file; include_file; include_file = include_file->next) {
 		fprintf(fp, "%s", include_file->name);
 		fputc('\0', fp);
-		}
+	}
 	fputc('\0', fp);
 	fclose(fp);
-	}
+}
 
 
 void write_obj_record(struct prog_info *pi, int address, int data)
-	{
+{
 	fputc((address >> 16) & 0xff, pi->obj_file);
 	fputc((address >> 8) & 0xff, pi->obj_file);
 	fputc(address & 0xff, pi->obj_file);
@@ -268,4 +317,7 @@ void write_obj_record(struct prog_info *pi, int address, int data)
 		fputc(1, pi->obj_file);
 	else
 		fputc(0, pi->obj_file);
-	}
+}
+
+/* end of file.c */
+
