@@ -1,6 +1,8 @@
 /***********************************************************************
+ *
  *  avra - Assembler for the Atmel AVR microcontroller series
- *  Copyright (C) 1998-2001 Jon Anders Haugum
+ *
+ *  Copyright (C) 1998-2004 Jon Anders Haugum, Tobias Weber
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,9 +20,9 @@
  *  Boston, MA 02111-1307, USA.
  *
  *
- *  Author of avra can be reached at:
- *     email: jonah@omegav.ntnu.no
- *     www: http://www.omegav.ntnu.no/~jonah/el/avra.html
+ *  Authors of avra can be reached at:
+ *     email: jonah@omegav.ntnu.no, tobiw@suprafluid.com
+ *     www: http://sourceforge.net/projects/avra
  */
 
 #include <stdio.h>
@@ -34,8 +36,7 @@
 
 #define MAX_MNEMONIC_LEN	8	// Maximum mnemonic length
 
-enum
-	{
+enum {
 	MNEMONIC_NOP = 0,  //          0000 0000 0000 0000
 	MNEMONIC_SEC,      //          1001 0100 0000 1000
 	MNEMONIC_CLC,      //          1001 0100 1000 1000
@@ -179,18 +180,16 @@ enum
 	MNEMONIC_STD_Y,    // Y+q, Rr  10q0 qq1r rrrr 1qqq
 	MNEMONIC_STD_Z,    // Z+q, Rr  10q0 qq1r rrrr 0qqq
 	MNEMONIC_END
-	};
+};
 
-struct instruction
-	{
+struct instruction {
 	char *mnemonic;
 	int opcode;
         int flag;	/* Device flags meaning the instruction is not
                            supported */
-	};
+};
 
-struct instruction instruction_list[] =
-	{
+struct instruction instruction_list[] = {
 	{"nop",   0x0000,          0},
 	{"sec",   0x9408,          0},
 	{"clc",   0x9488,          0},
@@ -337,238 +336,216 @@ struct instruction instruction_list[] =
 	};
 
 
-int parse_mnemonic(struct prog_info *pi, int pass)
-	{
-	int mnemonic, i, opcode, opcode2, instruction_long = False;
-	char *operand1, *operand2;
-	struct macro *macro;
-	char temp[MAX_MNEMONIC_LEN+1];
+/* We try to parse the command name. Is it a assembler mnemonic or anything else ?
+ * If so, it may be a macro.
+ */
 
-	operand1 = get_next_token(pi->fi->scratch, TERM_SPACE);
-	mnemonic = get_mnemonic_type(my_strlwr(pi->fi->scratch));
-	if(mnemonic == -1)
-		{
-		macro = get_macro(pi, pi->fi->scratch);
-		if(macro)
-			{
-			return(expand_macro(pi, macro, operand1, pass));
-			}
-		else
-			{
-			print_msg(pi, MSGTYPE_ERROR, "Unknown mnemonic/macro: %s", pi->fi->scratch);
-			return(True);
-			}
+int parse_mnemonic(struct prog_info *pi) 
+{
+  int mnemonic;
+  int i;
+  int opcode = 0;
+  int opcode2 = 0;
+  int instruction_long = False;
+  char *operand1;
+  char *operand2;
+  struct macro *macro;
+  char temp[MAX_MNEMONIC_LEN + 1];
+
+  operand1 = get_next_token(pi->fi->scratch, TERM_SPACE);  // we get the first word on line
+  mnemonic = get_mnemonic_type(my_strlwr(pi->fi->scratch));
+  if(mnemonic == -1) {// if -1 this must be a macro name
+	macro = get_macro(pi, pi->fi->scratch); // and so, we try to get the corresponding macro struct.
+	if(macro) {
+      return(expand_macro(pi, macro, operand1)); //we expand the macro
+	}
+	else { // if we cant find a name, this is a unknown word.
+      print_msg(pi, MSGTYPE_ERROR, "Unknown mnemonic/macro: %s", pi->fi->scratch);
+	  return(True);
+	}
+  }
+  if(pi->pass == PASS_2) {
+    if(mnemonic <= MNEMONIC_BREAK) {
+	  if(operand1) {
+		print_msg(pi, MSGTYPE_WARNING, "Garbage after instruction %s: %s", instruction_list[mnemonic].mnemonic, operand1);
+	  }
+	// No operand
+	opcode = 0;
+	}
+	else if(mnemonic <= MNEMONIC_ELPM) {
+      if(operand1) {
+		operand2 = get_next_token(operand1, TERM_COMMA);
+		if(!operand2) {
+		  print_msg(pi, MSGTYPE_ERROR, "%s needs a second operand", instruction_list[mnemonic].mnemonic);
+		  return(True);
 		}
-	if(pass == PASS_2)
-		{
-		if(mnemonic <= MNEMONIC_BREAK)
-			{
-			if(operand1)
-				print_msg(pi, MSGTYPE_WARNING, "Garbage after instruction %s: %s", instruction_list[mnemonic].mnemonic, operand1);
-			// No operand
-			opcode = 0;
-			}
-		else if(mnemonic <= MNEMONIC_ELPM)
-		{
-			if(operand1)
-			{
-				operand2 = get_next_token(operand1, TERM_COMMA);
-				if(!operand2)
-				{
-					print_msg(pi, MSGTYPE_ERROR, "%s needs a second operand", instruction_list[mnemonic].mnemonic);
-					return(True);
-				}
-				get_next_token(operand2, TERM_END);
-				i = get_register(pi, operand1);
-				opcode = i << 4;
-				i = get_indirect(pi, operand2);
-				if(i == 6) // Means Z
-				{
-					if(mnemonic == MNEMONIC_LPM)
-						mnemonic = MNEMONIC_LPM_Z;
-					else if(mnemonic == MNEMONIC_ELPM)
-						mnemonic = MNEMONIC_ELPM_Z;
-				}
-				else if(i == 7) // Means Z+
-				{
-					if(mnemonic == MNEMONIC_LPM)
-						mnemonic = MNEMONIC_LPM_ZP;
-					else if(mnemonic == MNEMONIC_ELPM)
-						mnemonic = MNEMONIC_ELPM_ZP;
-				}
-				else
-				{
-					print_msg(pi, MSGTYPE_ERROR, "Unsupported operand: %s", operand2);
-					return(True);
-				}
-			}
-			else
-				opcode = 0;
+		get_next_token(operand2, TERM_END);
+		i = get_register(pi, operand1);
+		opcode = i << 4;
+		i = get_indirect(pi, operand2);
+		if(i == 6) { // Means Z
+		  if(mnemonic == MNEMONIC_LPM)
+		    mnemonic = MNEMONIC_LPM_Z;
+		  else if(mnemonic == MNEMONIC_ELPM)
+			mnemonic = MNEMONIC_ELPM_Z;
 		}
-		else
-			{
-			if(!operand1)
-				{
-				print_msg(pi, MSGTYPE_ERROR, "%s needs an operand", instruction_list[mnemonic].mnemonic);
-				return(True);
-				}
-			operand2 = get_next_token(operand1, TERM_COMMA);
-			if(mnemonic >= MNEMONIC_BRBS)
-				{
-				if(!operand2)
-					{
-					print_msg(pi, MSGTYPE_ERROR, "%s needs a second operand", instruction_list[mnemonic].mnemonic);
-					return(True);
-					}
-				get_next_token(operand2, TERM_END);
-				}
-			if(mnemonic <= MNEMONIC_BCLR)
-				{
-				if(!get_bitnum(pi, operand1, &i))
-					return(False);
-				opcode = i << 4;
-				}
-			else if(mnemonic <= MNEMONIC_ROL)
-				{
-				i = get_register(pi, operand1);
-				if((mnemonic == MNEMONIC_SER) && (i < 16))
-					{
-					print_msg(pi, MSGTYPE_ERROR, "%s can only use a high register (r16 - r31)", instruction_list[mnemonic].mnemonic);
-					i &= 0x0f;
-					}
-				opcode = i << 4;
-				if(mnemonic >= MNEMONIC_TST)
-					opcode |= ((i & 0x10) << 5) | (i & 0x0f);
-				}
-			else if(mnemonic <= MNEMONIC_RCALL)
-				{
-				if(!get_expr(pi, operand1, &i))
-					return(False);
-				i -= pi->cseg_addr + 1;
-				if(mnemonic <= MNEMONIC_BRID)
-					{
-					if((i < -64) || (i > 63))
-						print_msg(pi, MSGTYPE_ERROR, "Branch out of range (-64 <= k <= 63)");
-					opcode = (i & 0x7f) << 3;
-					}
-				else
-					{
-					if(((i < -2048) || (i > 2047)) && (pi->device->flash_size != 4096))
-						print_msg(pi, MSGTYPE_ERROR, "Relative address out of range (-2048 <= k <= 2047)");
-					opcode = i & 0x0fff;
-					}
-				}
-			else if(mnemonic <= MNEMONIC_CALL)
-				{
-				if(!get_expr(pi, operand1, &i))
-					return(False);
-				if((i < 0) || (i > 4194303))
-					print_msg(pi, MSGTYPE_ERROR, "Address out of range (0 <= k <= 4194303)");
-				opcode = ((i & 0x3e0000) >> 13) | ((i & 0x01000) >> 16);
-				opcode2 = i & 0xffff;
-				instruction_long = True;
-				}
-			else if(mnemonic <= MNEMONIC_BRBC)
-				{
-				if(!get_bitnum(pi, operand1, &i))
-					return(False);
-				opcode = i;
-				if(!get_expr(pi, operand2, &i))
-					return(False);
-				i -= pi->cseg_addr + 1;
-				if((i < -64) || (i > 63))
-					print_msg(pi, MSGTYPE_ERROR, "Branch out of range (-64 <= k <= 63)");
-				opcode |= (i & 0x7f) << 3;
-				}
-			else if(mnemonic <= MNEMONIC_MUL)
-				{
-				i = get_register(pi, operand1);
-				opcode = i << 4;
-				i = get_register(pi, operand2);
-				opcode |= ((i & 0x10) << 5) | (i & 0x0f);
-				}
-			else if(mnemonic <= MNEMONIC_MOVW)
-			{
-				i = get_register(pi, operand1);
-				if((i % 2) == 1)
-					print_msg(pi, MSGTYPE_ERROR, "%s must use a even numbered register for Rd", instruction_list[mnemonic].mnemonic);
-				opcode = (i / 2) << 4;
-				i = get_register(pi, operand2);
-				if((i % 2) == 1)
-					print_msg(pi, MSGTYPE_ERROR, "%s must use a even numbered register for Rr", instruction_list[mnemonic].mnemonic);
-				opcode |= i / 2;
-			}
-			else if(mnemonic <= MNEMONIC_MULS)
-			{
-				i = get_register(pi, operand1);
-				if(i < 16)
-					print_msg(pi, MSGTYPE_ERROR, "%s can only use a high register (r16 - r31)", instruction_list[mnemonic].mnemonic);
-				opcode = (i & 0x0f) << 4;
-				i = get_register(pi, operand2);
-				if(i < 16)
-					print_msg(pi, MSGTYPE_ERROR, "%s can only use a high register (r16 - r31)", instruction_list[mnemonic].mnemonic);
-				opcode |= (i & 0x0f);
-			}
-			else if(mnemonic <= MNEMONIC_FMULSU)
-			{
-				i = get_register(pi, operand1);
-				if((i < 16) || (i >= 24))
-					print_msg(pi, MSGTYPE_ERROR, "%s can only use registers (r16 - r23)", instruction_list[mnemonic].mnemonic);
-				opcode = (i & 0x07) << 4;
-				i = get_register(pi, operand2);
-				if((i < 16) || (i >= 24))
-					print_msg(pi, MSGTYPE_ERROR, "%s can only use registers (r16 - r23)", instruction_list[mnemonic].mnemonic);
-				opcode |= (i & 0x07);
-			}
-			else if(mnemonic <= MNEMONIC_SBIW)
-				{
-				i = get_register(pi, operand1);
-				if(!((i == 24) || (i == 26) || (i == 28) || (i == 30)))
-					print_msg(pi, MSGTYPE_ERROR, "%s can only use registers R24, R26, R28 or R30", instruction_list[mnemonic].mnemonic);
-				opcode = ((i - 24) / 2) << 4;
-				if(!get_expr(pi, operand2, &i))
-					return(False);
-				if((i < 0) || (i > 63))
-					print_msg(pi, MSGTYPE_ERROR, "Constant out of range (0 <= k <= 63)");
-				opcode |= ((i & 0x30) << 2) | (i & 0x0f);
-				}
-			else if(mnemonic <= MNEMONIC_CBR)
-				{
-				i = get_register(pi, operand1);
-				if(i < 16)
-					print_msg(pi, MSGTYPE_ERROR, "%s can only use a high register (r16 - r31)", instruction_list[mnemonic].mnemonic);
-				opcode = (i & 0x0f) << 4;
-				if(!get_expr(pi, operand2, &i))
-					return(False);
-				if((i < -128) || (i > 255))
-					print_msg(pi, MSGTYPE_WARNING, "Constant out of range (-128 <= k <= 255). Will be masked");
-				if(mnemonic == MNEMONIC_CBR)
-					i = ~i;
-				opcode |= ((i & 0xf0) << 4) | (i & 0x0f);
-				}
-			else if(mnemonic <= MNEMONIC_BLD)
-				{
-				i = get_register(pi, operand1);
-				opcode = i << 4;
-				if(!get_bitnum(pi, operand2, &i))
-					return(False);
-				opcode |= i;
-				}
-			else if(mnemonic == MNEMONIC_IN)
-				{
-				i = get_register(pi, operand1);
-				opcode = i << 4;
-				if(!get_expr(pi, operand2, &i))
-					return(False);
-				if((i < 0) || (i > 63))
-					print_msg(pi, MSGTYPE_ERROR, "I/O out of range (0 <= P <= 63)");
-				opcode |= ((i & 0x30) << 5) | (i & 0x0f);
-				}
-			else if(mnemonic == MNEMONIC_OUT)
-				{
-				if(!get_expr(pi, operand1, &i))
-					return(False);
-				if((i < 0) || (i > 63))
+		else if(i == 7) { // Means Z+
+          if(mnemonic == MNEMONIC_LPM)
+			mnemonic = MNEMONIC_LPM_ZP;
+		  else if(mnemonic == MNEMONIC_ELPM)
+			mnemonic = MNEMONIC_ELPM_ZP;
+		}
+		else {
+		  print_msg(pi, MSGTYPE_ERROR, "Unsupported operand: %s", operand2);
+		  return(True);
+		}
+	  }
+	  else
+		opcode = 0;
+	}
+	else {
+	  if(!operand1) {
+		print_msg(pi, MSGTYPE_ERROR, "%s needs an operand", instruction_list[mnemonic].mnemonic);
+		return(True);
+	  }
+	  operand2 = get_next_token(operand1, TERM_COMMA);
+	  if(mnemonic >= MNEMONIC_BRBS)	{
+		if(!operand2) {
+          print_msg(pi, MSGTYPE_ERROR, "%s needs a second operand", instruction_list[mnemonic].mnemonic);
+		  return(True);
+		}
+		get_next_token(operand2, TERM_END);
+	  }
+	  if(mnemonic <= MNEMONIC_BCLR) {
+        if(!get_bitnum(pi, operand1, &i))
+		  return(False);
+	    opcode = i << 4;
+	  }
+	  else if(mnemonic <= MNEMONIC_ROL)	{
+		i = get_register(pi, operand1);
+		if((mnemonic == MNEMONIC_SER) && (i < 16)) {
+		  print_msg(pi, MSGTYPE_ERROR, "%s can only use a high register (r16 - r31)", instruction_list[mnemonic].mnemonic);
+		  i &= 0x0f;
+		}
+		opcode = i << 4;
+		if(mnemonic >= MNEMONIC_TST)
+		  opcode |= ((i & 0x10) << 5) | (i & 0x0f);
+	  }
+	  else if(mnemonic <= MNEMONIC_RCALL) {
+		if(!get_expr(pi, operand1, &i))
+		  return(False);
+		i -= pi->cseg_addr + 1;
+		if(mnemonic <= MNEMONIC_BRID) {
+		  if((i < -64) || (i > 63))
+			print_msg(pi, MSGTYPE_ERROR, "Branch out of range (-64 <= k <= 63)");
+		  opcode = (i & 0x7f) << 3;
+		}
+		else {
+		  if(((i < -2048) || (i > 2047)) && (pi->device->flash_size != 4096))
+			print_msg(pi, MSGTYPE_ERROR, "Relative address out of range (-2048 <= k <= 2047)");
+		  opcode = i & 0x0fff;
+		}
+	  }
+	  else if(mnemonic <= MNEMONIC_CALL) {
+	    if(!get_expr(pi, operand1, &i))
+		  return(False);
+		if((i < 0) || (i > 4194303))
+		  print_msg(pi, MSGTYPE_ERROR, "Address out of range (0 <= k <= 4194303)");	
+		opcode = ((i & 0x3e0000) >> 13) | ((i & 0x010000) >> 16);
+		opcode2 = i & 0xffff;
+		instruction_long = True;
+	  }
+	  else if(mnemonic <= MNEMONIC_BRBC) {
+		if(!get_bitnum(pi, operand1, &i))
+		  return(False);
+		opcode = i;
+		if(!get_expr(pi, operand2, &i))
+		  return(False);
+		i -= pi->cseg_addr + 1;
+		  if((i < -64) || (i > 63))
+		print_msg(pi, MSGTYPE_ERROR, "Branch out of range (-64 <= k <= 63)");
+		opcode |= (i & 0x7f) << 3;
+	  }
+	  else if(mnemonic <= MNEMONIC_MUL) {
+	    i = get_register(pi, operand1);
+		opcode = i << 4;
+		i = get_register(pi, operand2);
+		opcode |= ((i & 0x10) << 5) | (i & 0x0f);
+	  }
+	  else if(mnemonic <= MNEMONIC_MOVW) {
+	    i = get_register(pi, operand1);
+		if((i % 2) == 1)
+		  print_msg(pi, MSGTYPE_ERROR, "%s must use a even numbered register for Rd", instruction_list[mnemonic].mnemonic);
+		opcode = (i / 2) << 4;
+		i = get_register(pi, operand2);
+		if((i % 2) == 1)
+		  print_msg(pi, MSGTYPE_ERROR, "%s must use a even numbered register for Rr", instruction_list[mnemonic].mnemonic);
+		opcode |= i / 2;
+	  }
+	  else if(mnemonic <= MNEMONIC_MULS) {
+		i = get_register(pi, operand1);
+		if(i < 16)
+		  print_msg(pi, MSGTYPE_ERROR, "%s can only use a high register (r16 - r31)", instruction_list[mnemonic].mnemonic);
+		opcode = (i & 0x0f) << 4;
+		i = get_register(pi, operand2);
+		if(i < 16)
+		  print_msg(pi, MSGTYPE_ERROR, "%s can only use a high register (r16 - r31)", instruction_list[mnemonic].mnemonic);
+		opcode |= (i & 0x0f);
+	  }
+	  else if(mnemonic <= MNEMONIC_FMULSU) {
+		i = get_register(pi, operand1);
+		if((i < 16) || (i >= 24))
+		  print_msg(pi, MSGTYPE_ERROR, "%s can only use registers (r16 - r23)", instruction_list[mnemonic].mnemonic);
+		opcode = (i & 0x07) << 4;
+		i = get_register(pi, operand2);
+		if((i < 16) || (i >= 24))
+		  print_msg(pi, MSGTYPE_ERROR, "%s can only use registers (r16 - r23)", instruction_list[mnemonic].mnemonic);
+		opcode |= (i & 0x07);
+	  }
+	  else if(mnemonic <= MNEMONIC_SBIW) {
+		i = get_register(pi, operand1);
+		if(!((i == 24) || (i == 26) || (i == 28) || (i == 30)))
+		  print_msg(pi, MSGTYPE_ERROR, "%s can only use registers R24, R26, R28 or R30", instruction_list[mnemonic].mnemonic);
+		opcode = ((i - 24) / 2) << 4;
+		if(!get_expr(pi, operand2, &i))
+		  return(False);
+		if((i < 0) || (i > 63))
+		  print_msg(pi, MSGTYPE_ERROR, "Constant out of range (0 <= k <= 63)");
+		  opcode |= ((i & 0x30) << 2) | (i & 0x0f);
+		}
+		else if(mnemonic <= MNEMONIC_CBR) {
+		  i = get_register(pi, operand1);
+		  if(i < 16)
+		    print_msg(pi, MSGTYPE_ERROR, "%s can only use a high register (r16 - r31)", instruction_list[mnemonic].mnemonic);
+		  opcode = (i & 0x0f) << 4;
+		  if(!get_expr(pi, operand2, &i))
+		    return(False);
+		  if((i < -128) || (i > 255))
+		    print_msg(pi, MSGTYPE_WARNING, "Constant out of range (-128 <= k <= 255). Will be masked");
+		  if(mnemonic == MNEMONIC_CBR)
+		    i = ~i;
+		  opcode |= ((i & 0xf0) << 4) | (i & 0x0f);
+		}
+		else if(mnemonic <= MNEMONIC_BLD) {
+		  i = get_register(pi, operand1);
+		  opcode = i << 4;
+		  if(!get_bitnum(pi, operand2, &i))
+			return(False);
+		  opcode |= i;
+		}
+		else if(mnemonic == MNEMONIC_IN) {
+		  i = get_register(pi, operand1);
+		  opcode = i << 4;
+		  if(!get_expr(pi, operand2, &i))
+			return(False);
+		  if((i < 0) || (i > 63))
+			print_msg(pi, MSGTYPE_ERROR, "I/O out of range (0 <= P <= 63)");
+		  opcode |= ((i & 0x30) << 5) | (i & 0x0f);
+		}
+		else if(mnemonic == MNEMONIC_OUT) {
+		  if(!get_expr(pi, operand1, &i))
+		    return(False);
+		  if((i < 0) || (i > 63))
 					print_msg(pi, MSGTYPE_ERROR, "I/O out of range (0 <= P <= 63)");
 				opcode = ((i & 0x30) << 5) | (i & 0x0f);
 				i = get_register(pi, operand2);
@@ -678,9 +655,9 @@ int parse_mnemonic(struct prog_info *pi, int pass)
 		if(pi->list_on && pi->list_line)
 			{
 			if(instruction_long)
-				fprintf(pi->list_file, "%06x %04x %04x %s", pi->cseg_addr, opcode, opcode2, pi->list_line);
+				fprintf(pi->list_file, "C:%06x %04x %04x %s", pi->cseg_addr, opcode, opcode2, pi->list_line);
 			else
-				fprintf(pi->list_file, "%06x %04x      %s", pi->cseg_addr, opcode, pi->list_line);
+				fprintf(pi->list_file, "C:%06x %04x      %s", pi->cseg_addr, opcode, pi->list_line);
 			pi->list_line = NULL;
 			}
 		if(pi->hfi)
@@ -696,34 +673,35 @@ int parse_mnemonic(struct prog_info *pi, int pass)
 		}
 	else
 		{
-		if((mnemonic == MNEMONIC_JMP) || (mnemonic == MNEMONIC_CALL) || (mnemonic == MNEMONIC_LDS) || (mnemonic == MNEMONIC_STS))
-			{
+		if((mnemonic == MNEMONIC_JMP) || (mnemonic == MNEMONIC_CALL) 
+            || (mnemonic == MNEMONIC_LDS) || (mnemonic == MNEMONIC_STS)) {
 			pi->cseg_addr += 2;
 			pi->cseg_count += 2;
-			}
-		else
-			{
+		}
+		else {
 			pi->cseg_addr++;
 			pi->cseg_count++;
-			}
 		}
-	return(True);
 	}
+	return(True);
+}
 
 
 int get_mnemonic_type(char *mnemonic)
-	{
+{
 	int i;
 
-	for(i = 0; i < MNEMONIC_COUNT; i++)
-		if(!strcmp(mnemonic, instruction_list[i].mnemonic))
+	for(i = 0; i < MNEMONIC_COUNT; i++) {
+		if(!strcmp(mnemonic, instruction_list[i].mnemonic)) {
 			return(i);
-	return(-1);
+		}
 	}
+	return(-1);
+}
 
 
 int get_register(struct prog_info *pi, char *data)
-	{
+{
 	char *second_reg;
 	int reg = 0;
 	struct def *def;
@@ -739,43 +717,39 @@ int get_register(struct prog_info *pi, char *data)
 			reg = def->reg;
 			return(reg);
 			}
-	if((tolower(data[0]) == 'r') && isdigit(data[1]))
-		{
+	if((tolower(data[0]) == 'r') && isdigit(data[1])) {
 		reg = atoi(&data[1]);
 		if(reg > 31)
 			print_msg(pi, MSGTYPE_ERROR, "R%d is not a valid register", reg);
-		}
+	}
 	else
 		print_msg(pi, MSGTYPE_ERROR, "No register associated with %s", data);
 	return(reg);
-	}
+}
 
 
 int get_bitnum(struct prog_info *pi, char *data, int *ret)
 {
 	if(!get_expr(pi, data, ret))
 		return(False);
-	if((*ret < 0) || (*ret > 7))
-		{
+	if((*ret < 0) || (*ret > 7)) {
 		print_msg(pi, MSGTYPE_ERROR, "Operand out of range (0 <= s <= 7)");
 		return(False);
-		}
-	return(True);
 	}
+	return(True);
+}
 
 
 int get_indirect(struct prog_info *pi, char *operand)
-	{
+{
 	int i = 1;
 
-	switch(tolower(operand[0]))
-		{
+	switch(tolower(operand[0])) {
 		case '-':
 			while(IS_HOR_SPACE(operand[i])) i++;
 			if(operand[i + 1] != '\0')
 				print_msg(pi, MSGTYPE_ERROR, "Garbage in operand (%s)", operand);
-			switch(tolower(operand[i]))
-				{
+			switch(tolower(operand[i])) {
 				case 'x':
         	   	                if (pi->device->flag & DF_NO_XREG)
                                            print_msg(pi, MSGTYPE_ERROR, "X register is not supported on %s", pi->device->name);
@@ -789,17 +763,16 @@ int get_indirect(struct prog_info *pi, char *operand)
 				default:
 					print_msg(pi, MSGTYPE_ERROR, "Garbage in operand (%s)", operand);
 					return(0);
-				}
+			}
 		case 'x':
                         if (pi->device->flag & DF_NO_XREG)
                            print_msg(pi, MSGTYPE_ERROR, "X register is not supported on %s", pi->device->name);
 			while(IS_HOR_SPACE(operand[i])) i++;
-			if(operand[i] == '+')
-				{
+			if(operand[i] == '+') {
 				if(operand[i + 1] != '\0')
 					print_msg(pi, MSGTYPE_ERROR, "Garbage in operand (%s)", operand);
 				return(1);
-				}
+			}
 			else if(operand[i] == '\0')
 				return(0);
 			else
@@ -809,12 +782,11 @@ int get_indirect(struct prog_info *pi, char *operand)
                         if (pi->device->flag & DF_NO_YREG)
                            print_msg(pi, MSGTYPE_ERROR, "Y register is not supported on %s", pi->device->name);
 			while(IS_HOR_SPACE(operand[i])) i++;
-			if(operand[i] == '+')
-				{
+			if(operand[i] == '+') {
 				if(operand[i + 1] != '\0')
 					print_msg(pi, MSGTYPE_ERROR, "Garbage in operand (%s)", operand);
 				return(4);
-				}
+			}
 			else if(operand[i] == '\0')
 				return(3);
 			else
@@ -822,12 +794,11 @@ int get_indirect(struct prog_info *pi, char *operand)
 			return(0);
 		case 'z':
 			while(IS_HOR_SPACE(operand[i])) i++;
-			if(operand[i] == '+')
-				{
+			if(operand[i] == '+') {
 				if(operand[i + 1] != '\0')
 					print_msg(pi, MSGTYPE_ERROR, "Garbage in operand (%s)", operand);
 				return(7);
-				}
+			}
 			else if(operand[i] == '\0')
 				return(6);
 			else
@@ -835,9 +806,9 @@ int get_indirect(struct prog_info *pi, char *operand)
 			return(0);
 		default:
 			print_msg(pi, MSGTYPE_ERROR, "Garbage in operand (%s)", operand);
-		}
-	return(0);
 	}
+	return(0);
+}
 
 /* Return 1 if instruction name is supported by the current device,
    0 if unsupported, -1 if it is invalid */
@@ -850,13 +821,12 @@ int is_supported(struct prog_info *pi, char *name) {
    if (mnemonic == -1) return -1;
    if (pi->device->flag & instruction_list[mnemonic].flag) return 0;
    return 1;
-   }
+}
 
 int count_supported_instructions(int flags)
 {
   int i = 0, count = 0;
-  while(i < MNEMONIC_END)
-  {
+  while(i < MNEMONIC_END) {
     if((i < MNEMONIC_LD) || (i > MNEMONIC_COUNT))
       if(!(flags & instruction_list[i].flag))
         count++;
@@ -864,3 +834,6 @@ int count_supported_instructions(int flags)
   }
   return(count);
 }
+
+/* end of mnemonic.c */
+
